import virustotal_python
import pytest
import os.path
from time import sleep
from base64 import urlsafe_b64encode

# Declare variables for tests
# Create dictionary containing the file to send for multipart encoding upload
FILES = {
    "file": (
        os.path.basename("virustotal_python/oldexamples.py"),
        open(os.path.abspath("virustotal_python/oldexamples.py"), "rb"),
    )
}
# The ID (either SHA-256, SHA-1 or MD5) identifying the file
FILE_ID = "9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115"
# Example IP address (Google DNS)
IP = "8.8.8.8"
# Example ID of a graph
## NOTE: There are no comments on this graph so an empty list is returned
GRAPH_ID = "g70fae134aefc4e2f90f069aba47d15a92e0073564310443aa0b6ca3384f5240d"
# URL/domain identifier
URL_DOMAIN = "google.com"
# Example comment ID
COMMENT_ID = "f-9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115-07457619"


@pytest.fixture()
def vtotal_v2(request):
    yield virustotal_python.Virustotal()

    def fin():
        """
        Helper function which sleeps for 15 seconds between each test. This is to avoid VirusTotal 403 rate quota limits.
        """
        print("Sleeping for 15 seconds to avoid VirusTotal 403 rate quota limits...")
        sleep(15)

    request.addfinalizer(fin)


@pytest.fixture()
def vtotal_v3(request):
    yield virustotal_python.Virustotal(API_VERSION="v3")

    def fin():
        """
        Helper function which sleeps for 15 seconds between each test. This is to avoid VirusTotal 403 rate quota limits.
        """
        print("Sleeping for 15 seconds to avoid VirusTotal 403 rate quota limits...")
        sleep(15)

    request.addfinalizer(fin)


def test_file_scan_v2(vtotal_v2):
    """
    Test for sending a file to the VirusTotal v2 API for analysis.
    """
    # Create dictionary containing the file to send for multipart encoding upload
    files = {
        "file": (
            os.path.basename("virustotal_python/oldexamples.py"),
            open(os.path.abspath("virustotal_python/oldexamples.py"), "rb"),
        )
    }
    resp = vtotal_v2.request("file/scan", files=FILES, method="POST")
    data = resp.json()
    assert resp.response_code == 1
    assert data["scan_id"]
    assert data["permalink"]


def test_file_scan_v3(vtotal_v3):
    """
    Test for sending a file to the VirusTotal v3 API for analysis.
    """
    resp = vtotal_v3.request("files", files=FILES, method="POST")
    assert resp.status_code == 200
    data = resp.data
    assert data["id"]
    assert data["type"] == "analysis"


def test_file_info_v2(vtotal_v2):
    """
    Test for retrieving information about a file from the VirusTotal v2 API.
    """
    resp = vtotal_v2.request("file/report", {"resource": FILE_ID})
    assert resp.response_code == 1
    assert resp.json()["scans"]


def test_file_info_v3(vtotal_v3):
    """
    Test for retrieving information about a file from the VirusTotal v3 API.
    """
    resp = vtotal_v3.request(f"files/{FILE_ID}")
    assert resp.status_code == 200
    assert resp.object_type == "file"
    assert resp.data["attributes"]
    assert resp.data["attributes"]["last_analysis_results"]


def test_compatibility():
    """
    Test COMPATIBILITY_ENABLED parameter on Virustotal class.
    """
    vtotal = virustotal_python.Virustotal(API_VERSION="v3", COMPATIBILITY_ENABLED=True)
    resp = vtotal.request(f"files/{FILE_ID}")
    assert resp["status_code"] == 200
    assert resp["json_resp"]["data"]["type"] == "file"
    assert resp["json_resp"]["data"]["attributes"]


def test_scan_url_info_v2(vtotal_v2):
    """
    Test scanning URL and retrieving the scan results from the VirusTotal v2 API.
    """
    # Send the URLs to VirusTotal for analysis
    resp = vtotal_v2.request("url/scan", params={"url": URL_DOMAIN}, method="POST")
    assert resp.status_code == 200
    data = resp.json()
    # Obtain scan_id
    scan_id = data["scan_id"]
    # Request report for URL analysis
    analysis_resp = vtotal_v2.request("url/report", params={"resource": scan_id})
    assert analysis_resp.status_code == 200
    assert analysis_resp.response_code == 1
    data = analysis_resp.json()
    assert data["scan_id"]
    assert data["verbose_msg"]
    assert data["url"] == f"http://{URL_DOMAIN}/"
    assert data["scan_date"]


def test_scan_url_info_v3(vtotal_v3):
    """
    Test scanning URL and retrieving the scan results from the VirusTotal v3 API.
    """
    resp = vtotal_v3.request("urls", data={"url": URL_DOMAIN}, method="POST")
    assert resp.status_code == 200
    assert resp.data["id"]
    # URL safe encode URL in base64 format
    # https://developers.virustotal.com/v3.0/reference#url
    url_id = urlsafe_b64encode(URL_DOMAIN.encode()).decode().strip("=")
    print(f"URL: {URL_DOMAIN} ID: {url_id}")
    # Obtain the analysis results for the URL using the url_id
    analysis_resp = vtotal_v3.request(f"urls/{url_id}")
    assert analysis_resp.status_code == 200
    assert analysis_resp.object_type == "url"
    assert analysis_resp.data["attributes"]


def test_domain_info_v2(vtotal_v2):
    """
    Test for retrieving domain information from the VirusTotal v2 API.
    """
    resp = vtotal_v2.request("domain/report", params={"domain": URL_DOMAIN})
    assert resp.response_code == 1
    json = resp.json()
    assert json["Alexa rank"]
    assert json["Alexa domain info"]
    assert json["Webutation domain info"]["Verdict"]
    assert json["whois_timestamp"]


def test_domain_info_v3(vtotal_v3):
    """
    Test for retrieving domain information from the VirusTotal v3 API.
    """
    resp = vtotal_v3.request(f"domains/{URL_DOMAIN}")
    assert resp.status_code == 200
    assert resp.object_type == "domain"
    data = resp.data
    assert isinstance(data["links"], dict)
    assert data["attributes"]["last_analysis_results"]
    assert data["attributes"]["creation_date"]


def test_retrieve_comment_file_id_v3(vtotal_v3):
    """
    Test for retrieving a comment for a given file ID.
    """
    resp = vtotal_v3.request(f"files/{FILE_ID}/comments", params={"limit": 2})
    assert resp.status_code == 200
    assert resp.links
    assert resp.meta
    assert resp.cursor
    json = resp.data
    # Retrieve first comment text
    assert json[0]["attributes"]["text"]
    # Retrieve second comment tags
    assert json[1]["attributes"]["votes"] and isinstance(
        json[1]["attributes"]["votes"], dict
    )


def test_retrieve_comment_file_id_v2(vtotal_v2):
    """
    Test for retrieving a comment for a given file ID using the VirusTotal v2 API.
    """
    resp = vtotal_v2.request("comments/get", params={"resource": FILE_ID})
    assert resp.status_code == 200
    assert resp.response_code == 1
    json = resp.json()
    for commentdata in json["comments"]:
        assert commentdata["date"]
        assert commentdata["comment"]


def test_retrieve_comment_latest_v3(vtotal_v3):
    """
    Test for retrieveing the latest 10 comments made on VirusTotal.
    """
    resp = vtotal_v3.request("comments", params={"limit": 10})
    assert resp.status_code == 200
    assert resp.links
    assert resp.meta
    assert resp.cursor
    json = resp.data
    assert json[0]["attributes"]["date"]
    assert len(json) == 10


def test_retrieve_ip_info_v2(vtotal_v2):
    """
    Test for retrieving information about an IP address using the VirusTotal v2 API.
    """
    resp = vtotal_v2.request("ip-address/report", params={"ip": IP})
    assert resp.status_code == 200
    assert resp.response_code == 1
    json = resp.json()
    assert json["as_owner"] == "GOOGLE"
    assert json["country"] == "US"
    assert json["verbose_msg"] == "IP address in dataset"
    for sample in json["detected_communicating_samples"]:
        assert sample["date"]
        assert sample["positives"]
        assert sample["sha256"]
        assert sample["total"]


def test_retrieve_ip_info_v3(vtotal_v3):
    """
    Test for retrieving information about an IP address.
    """
    resp = vtotal_v3.request(f"ip_addresses/{IP}")
    assert resp.status_code == 200
    data = resp.data
    assert data["attributes"]["as_owner"] == "GOOGLE"
    assert data["attributes"]["country"] == "US"
    assert data["attributes"]["last_analysis_stats"]
    assert data["attributes"]["reputation"]
    assert resp.object_type == "ip_address"


def test_retrieve_graph_v3(vtotal_v3):
    """
    Test for retrieving the latest 3 graphs made on VirusTotal.
    """
    resp = vtotal_v3.request("graphs", params={"limit": 3})
    assert resp.status_code == 200
    assert len(resp.data) == 3
    data = resp.data
    assert data[0]["attributes"]["graph_data"]
    assert resp.links
    assert resp.meta
    assert resp.cursor


def test_search_v3(vtotal_v3):
    """
    Test searching the VirusTotal API.
    """
    resp = vtotal_v3.request("search", params={"query": URL_DOMAIN})
    assert resp.status_code == 200
    assert resp.links
    assert resp.object_type
    data = resp.data
    assert data[0]["attributes"]["creation_date"]
    assert data[0]["attributes"]["last_analysis_results"]
    assert data[0]["attributes"]["last_analysis_stats"]
    assert data[0]["attributes"]["last_dns_records"]
    assert data[0]["attributes"]["last_https_certificate"]
    assert data[0]["attributes"]["total_votes"]


def test_metadata_v3(vtotal_v3):
    """
    Test retrieving metadata from the VirusTotal API.
    """
    resp = vtotal_v3.request("metadata")
    assert resp.status_code == 200
    engines_dict = resp.data["engines"]
    assert engines_dict.keys()
    assert "Microsoft" in engines_dict.keys()


def test_contextmanager_v2():
    """
    Test basic Context Manager support.
    """
    with virustotal_python.Virustotal() as vtotal:
        # Retrieve information about an IP address
        resp = vtotal.request("ip-address/report", params={"ip": IP})
        assert resp.status_code == 200
        data = resp.json()
        assert data["as_owner"] == "GOOGLE"
        assert data["country"] == "US"


def test_contextmanager_v3():
    """
    Test basic Context Manager support.
    """
    with virustotal_python.Virustotal(API_VERSION="v3") as vtotal:
        # Retrieve information about an IP address
        resp = vtotal.request(f"ip_addresses/{IP}")
        assert resp.status_code == 200
        data = resp.data
        assert data["id"] == IP
        assert data["attributes"]["as_owner"] == "GOOGLE"
        assert data["attributes"]["country"] == "US"



"""
The examples below are for virustotal-python versions <0.1.0.

Examples for versions >=0.1.0 head to /examples directory.
"""
from virustotal import Virustotal
from pprint import pprint
import os.path

# Normal Initialisation
vtotal = Virustotal("Insert API key here.")

# NEW as of version 0.0.5: Proxy support
# Example Usage: Using HTTP(S)
vtotal = Virustotal(
    "Insert API key here.",
    {"http": "http://10.10.1.10:3128", "https": "http://10.10.1.10:1080"},
)
# Or using SOCKS
vtotal = Virustotal(
    "Insert API key here.",
    {"http": "socks5://user:pass@host:port", "https": "socks5://user:pass@host:port"},
)

# NOTE: Check virustotal.py for docstrings containing full parameter descriptions

# Send a file to Virustotal for analysis
resp = vtotal.file_scan("./tests.py")  # PATH to file for querying

# NOTE: This endpoint has been removed from the Public Virustotal API
# Resend a file to Virustotal for analysis
# A list containing the resource (SHA256) HASH of the file above
# resp = vtotal.file_rescan(
#    ["75efd85cf6f8a962fe016787a7f57206ea9263086ee496fc62e3fc56734d4b53"]
# )
## A list containing md5/sha1/sha256 hashes. Can be a combination of any of the three allowed hashes (MAX 25 items)
## NOTE: The second hash here is flagged as malicious by multiple engines
# resp = vtotal.file_rescan(
#    [
#        "75efd85cf6f8a962fe016787a7f57206ea9263086ee496fc62e3fc56734d4b53",
#        "9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115",
#    ]
# )

# Retrieve scan report(s) for given file(s) from Virustotal
# A list containing the resource (SHA256) HASH of a known malicious file
resp = vtotal.file_report(
    ["9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115"]
)
# A list of resource(s). Can be `md5/sha1/sha256 hashes` and/or combination of hashes and scan_ids (MAX 4 per standard request rate)
# The first is a scan_id, the second is a SHA256 HASH
resp = vtotal.file_report(
    [
        "75efd85cf6f8a962fe016787a7f57206ea9263086ee496fc62e3fc56734d4b53-1555351539",
        "9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115",
    ]
)

# Query url(s) to VirusTotal
# A list containing a url to be scanned by VirusTotal
resp = vtotal.url_scan(["ihaveaproblem.info"])  # Query a single url
# A list of url(s) to be scanned by VirusTotal (MAX 4 per standard request rate)
resp = vtotal.url_scan(
    ["ihaveaproblem.info", "google.com", "wikipedia.com", "github.com"]
)

# Retrieve url report(s)
# A list containing the url of the report to be retrieved
resp = vtotal.url_report(["ihaveaproblem.info"]) # Query a single url
# A list of the url(s) and/or scan_id(s) report(s) to be retrieved (MAX 4 per standard request rate)
# The first object in the list is a scan_id
resp = vtotal.url_report(
    [
        "fd21590d9df715452c8c000e1b5aa909c7c5ea434c2ddcad3f4ccfe9b0ee224e-1555352750",
        "google.com",
        "wikipedia.com",
        "github.com",
    ],
    scan=1,
)

# Query an IP to Virustotal
resp = vtotal.ipaddress_report("90.156.201.27")

# Retrieve a domain report
resp = vtotal.domain_report("027.ru")

# Put a comment onto a specific resource
resp = vtotal.put_comment(
    "9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115",
    comment="#watchout, this looks very malicious!",
)

pprint(resp)



"""
MIT License

Copyright (c) 2021 dbrennand

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import requests
import os
from typing import Tuple
from json.decoder import JSONDecodeError


class VirustotalError(Exception):
    """
    Class for VirusTotal API errors.
    """

    def __init__(self, response: requests.Response):
        """
        Initalisation for VirustotalError class.

        :param response: A requests.Response object from a failed API request to the VirusTotal API.
        """
        self.response = response

    def __str__(self):
        return f"Error {self.error().get('code', 'unknown')} ({self.response.status_code}): {self.error().get('message', 'No message')}"

    def error(self) -> dict:
        """
        Retrieve the error that occurred from a VirusTotal API request.

        [v3 documentation](https://developers.virustotal.com/v3.0/reference#errors)

        [v2 documentation](https://developers.virustotal.com/reference#api-responses)

        :returns: A dictionary containing the error code and message returned from the VirusTotal API (if any) otherwise, returns an empty dictionary.
        """
        # Attempt to decode JSON as the v3 VirusTotal API returns the error message as JSON
        try:
            return self.response.json().get("error", dict())
        except ValueError:
            # Catch exception if there is no JSON to be deserialized
            # Most likely using the v2 VirusTotal API
            # Check there is response text, if not, return an empty dict
            if self.response.text:
                return dict(message=self.response.text)
            else:
                return dict()


class VirustotalResponse(object):
    """
    Response class for VirusTotal API requests.
    """

    def __init__(self, response: requests.Response):
        """
        Initalisation for VirustotalResponse class.

        :param response: A requests.Response object from a successfull API request to the VirusTotal API.
        """
        self.response = response

    @property
    def headers(self) -> dict:
        """
        Retrieve the HTTP headers of a VirusTotal API request.

        :returns: The HTTP headers of the requests.Response object.
        """
        return self.response.headers

    @property
    def status_code(self) -> int:
        """
        Retrieve the HTTP status code of a VirusTotal API request.

        :returns: The HTTP status code of the requests.Response object.
        """
        return self.response.status_code

    @property
    def text(self) -> str:
        """
        Retrieve the HTTP text response of a VirusTotal API request.

        :returns: The HTTP text response of the requests.Response object.
        """
        return self.response.text

    @property
    def requests_response(self) -> requests.Response:
        """
        Retrieve the HTTP requests.Response object of a VirusTotal API request.
        You may want to access this property if you wanted to read other aspects of the response such as cookies.

        :returns: A requests.Response object.
        """
        return self.response

    @property
    def links(self) -> Tuple[dict, None]:
        """
        Retrieve the value of the key 'links' in the JSON response from a VirusTotal API request.

        NOTE: Links are not retrieved for objects inside 'data'.

        [v3 documentation](https://developers.virustotal.com/v3.0/reference#collections)

        :returns: A dictionary containing the links used to retrieve the next set of objects (if any), otherwise, returns None.
        """
        return self.json().get("links", None)

    @property
    def meta(self) -> Tuple[dict, None]:
        """
        Retrieve the value of the key 'meta' in the JSON response from a VirusTotal API request.

        [v3 documentation](https://developers.virustotal.com/v3.0/reference#collections)

        :returns: A dictionary containing metadata about the object(s) (if any), otherwise, returns None.
        """
        return self.json().get("meta", None)

    @property
    def cursor(self) -> Tuple[str, None]:
        """
        Retrieve the value of the key 'cursor' in the JSON response value 'meta' from a VirusTotal API request.

        [v3 documentation](https://developers.virustotal.com/v3.0/reference#collections)

        :returns: A string representing the cursor used to retrieve additional related object(s), otherwise, returns None.
        """
        try:
            return self.meta.get("cursor", None)
        # Catch AttributeError that occurs when attemping to call attribute 'get' on None
        # which is returned if the 'meta' key is not present in the JSON response
        except AttributeError:
            return None

    @property
    def data(self) -> Tuple[dict, list, None]:
        """
        Retrieve the value of the key 'data' in the JSON response from a VirusTotal API request.

        [v3 documentation](https://developers.virustotal.com/v3.0/reference#objects)

        :returns: A dictionary or list depending on the number of objects returned from the VirusTotal API (if any) otherwise, returns None.
        """
        return self.json().get("data", None)

    @property
    def object_type(self) -> Tuple[list, str, None]:
        """
        Retrieve the object type(s) in the JSON response from a VirusTotal API request.

        [v3 documentation](https://developers.virustotal.com/v3.0/reference#objects)

        [More v3 documentation](https://developers.virustotal.com/v3.0/reference#collections)

        :returns: A list or string depending on the number of objects returned from the VirusTotal API (if any) otherwise, returns None.
        """
        data = self.data
        # Check if data is more than one object
        if isinstance(data, list):
            object_list = []
            for data_object in data:
                data_object_type = data_object.get("type", None)
                object_list.append(data_object_type)
            return object_list
        elif isinstance(data, dict):
            return data.get("type", None)
        else:
            return None

    @property
    def response_code(self) -> Tuple[int, None]:
        """
        Retrieve the value of the key 'response_code' in the JSON response from a VirusTotal v2 API request.

        [v2 documentation](https://developers.virustotal.com/reference#api-responses)

        :returns: An int of the response_code from the VirusTotal API (if any), otherwise, returns None.
        """
        return self.json().get("response_code", None)

    def json(self, **kwargs) -> Tuple[dict, list]:
        """
        Retrieve the JSON response of a VirusTotal API request.

        :param **kwargs: Parameters to pass to json. Identical to `json.loads(**kwargs)`.
        :returns: JSON response of the requests.Response object.
        :raises ValueError: Raises ValueError when the response body contains invalid JSON.
        """
        try:
            return self.response.json(**kwargs)
        except JSONDecodeError:
            return dict()


class Virustotal(object):
    """
    Interact with the public VirusTotal v2 and v3 APIs.

    [v2 documentation](https://www.virustotal.com/en/documentation/public-api/)

    [v3 documentation](https://developers.virustotal.com/v3.0/reference)
    """

    def __init__(
        self,
        API_KEY: str = os.environ.get("VIRUSTOTAL_API_KEY", None),
        API_VERSION: str = "v2",
        COMPATIBILITY_ENABLED: bool = False,
        PROXIES: dict = None,
        TIMEOUT: float = None,
    ):
        """
        Initalisation function for the Virustotal class.

        :param API_KEY: The API key used to interact with the VirusTotal v2 and v3 APIs. Alternatively, the environment variable `VIRUSTOTAL_API_KEY` can be provided.
        :param API_VERSION: The version to use when interacting with the VirusTotal API. This parameter defaults to 'v2' for backwards compatibility.
        :param COMPATIBILITY_ENABLED: Preserve the old response format of virustotal-python versions prior to 0.1.0 for backwards compatibility.
        :param PROXIES: A dictionary containing proxies used when making requests.
        :param TIMEOUT: A float for the amount of time to wait in seconds for the HTTP request before timing out.
        :raises ValueError: Raises ValueError when no API_KEY is provided or the API_VERSION is invalid.
        """
        self.VERSION = "0.1.3"
        if API_KEY is None:
            raise ValueError(
                "An API key is required to interact with the VirusTotal API.\nProvide one to the API_KEY parameter or by setting the environment variable 'VIRUSTOTAL_API_KEY'."
            )
        self.API_KEY = API_KEY
        self.COMPATIBILITY_ENABLED = COMPATIBILITY_ENABLED
        self.PROXIES = PROXIES
        self.TIMEOUT = TIMEOUT
        # Declare appropriate variables depending on the API_VERSION provided
        if API_VERSION == "v2":
            self.API_VERSION = API_VERSION
            self.BASEURL = "https://www.virustotal.com/vtapi/v2/"
            self.HEADERS = {
                "Accept-Encoding": "gzip, deflate",
                "User-Agent": f"gzip,  virustotal-python {self.VERSION}",
            }
        elif API_VERSION == "v3":
            self.API_VERSION = API_VERSION
            self.BASEURL = "https://www.virustotal.com/api/v3/"
            self.HEADERS = {
                "Accept-Encoding": "gzip, deflate",
                "User-Agent": f"gzip, virustotal-python {self.VERSION}",
                "x-apikey": f"{self.API_KEY}",
            }
        else:
            raise ValueError(
                f"The API version '{API_VERSION}' is not a valid VirusTotal API version.\nValid API versions are 'v2' or 'v3'."
            )

    # Context Manager support
    def __enter__(self):
        """
        Context Manager enter function.
        """
        return self

    def __exit__(self, type, value, traceback):
        """
        Context Manager exit function.
        """
        return

    def request(
        self,
        resource: str,
        params: dict = {},
        data: dict = None,
        json: dict = None,
        files: dict = None,
        method: str = "GET",
    ) -> Tuple[dict, VirustotalResponse]:
        """
        Make a request to the VirusTotal API.

        :param resource: A valid VirusTotal API endpoint. (E.g. 'files/{id}')
        :param params: A dictionary containing API endpoint query parameters.
        :param data: A dictionary containing the data to send in the body of the request.
        :param json: A dictionary containing the JSON payload to send with the request.
        :param files: A dictionary containing the file for multipart encoding upload. (E.g: {'file': ('filename', open('filename.txt', 'rb'))})
        :param method: The request method to use.
        :returns: A dictionary containing the HTTP response code (resp_code) and JSON response (json_resp) if self.COMPATIBILITY_ENABLED is True.
            Otherwise, a VirustotalResponse class object is returned. If a HTTP status not equal to 200 occurs. Then a VirustotalError class object is returned.
        :raises Exception: Raise Exception when an unsupported method is provided.
        """
        # Create API endpoint
        endpoint = f"{self.BASEURL}{resource}"
        # If API version being used is v2, add the API key to params
        if self.API_VERSION == "v2":
            params["apikey"] = self.API_KEY
        if method == "GET":
            response = requests.get(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        elif method == "POST":
            response = requests.post(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        elif method == "PATCH":
            response = requests.patch(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        elif method == "DELETE":
            response = requests.delete(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        else:
            raise Exception(f"The request method '{method}' is not supported.")
        # Validate response and return it
        return self.validate_response(response)

    def validate_response(
        self, response: requests.Response
    ) -> Tuple[dict, VirustotalResponse]:
        """
        Helper function to validate the request response.

        :param response: A requests.Response object for an API request made to the VirusTotal API.
        :returns: A dictionary containing the HTTP response code (resp_code) and JSON response (json_resp) if self.COMPATIBILITY_ENABLED is True otherwise, a VirustotalResponse class object is returned.
        :raises VirustotalError: Raises VirustotalError when an HTTP status code other than 200 (successfull) occurs.
        """
        if self.COMPATIBILITY_ENABLED:
            if response.status_code == 200:
                json_resp = response.json()
                return dict(status_code=response.status_code, json_resp=json_resp)
            else:
                # An error has occurred
                # The v3 API returns the error as JSON, attempt to retrieve it
                try:
                    error_json = response.json()
                except ValueError:
                    # API version being used is likely to be v2. Catch the raised ValueError and continue
                    pass
                return dict(
                    status_code=response.status_code,
                    # Provide JSON error message if retrieved successfully, otherwise fallback on response.text
                    error=(error_json if error_json else response.text),
                    resp=response.content,
                )
        else:
            if response.status_code != 200:
                raise VirustotalError(response)
            else:
                return VirustotalResponse(response)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf8") as readme:
    long_description = readme.read()

setup(
    name="virustotal-python",
    version="0.1.3",
    author="dbrennand",
    description="A Python library to interact with the public VirusTotal v2 and v3 APIs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dbrennand/virustotal-python",
    packages=find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="VirusTotal Wrapper Public API Library v2 v3",
)
